/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.*;
import eos.models.*;
import eos.models.Baseline2Model.Baseline2Farm;

/**
 * Uses a supply curve system to dynamically adjust the price of food.
 * 
 * @author Michael Adelson
 */
public class BL2FoodSupplyCurveFarm extends Baseline2Farm {

    /**
     * @param economy
     * @param goods
     */
    public BL2FoodSupplyCurveFarm(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.models.Baseline2Model.Baseline2Farm#baseLine2FarmSubclassAct()
     */
    protected void baseLine2FarmSubclassAct() {
        // buy labor normally
        this.laborMarket.addBuyOffer(this, Baseline2Model.MAX_PRODUCTION
                / Baseline2Model.LABOR_PRODUCTIVITY,
                Baseline2Model.HISTORICAL_LABOR_PRICE);

        // initially sell at mp
        if (this.foodMarket.getLastMarketPrice() <= 0.0) {
            if (this.food.getQuantity() > 0.0)
                this.foodMarket.addSellOffer(this, this.food.getQuantity(),
                        Baseline2Model.HISTORICAL_FOOD_PRICE);
            return;
        }

        // sell food using supply curve method assumes a linear production
        // function
        double quantity = this.food.getQuantity();
        double minPrice = Baseline2Model.HISTORICAL_LABOR_PRICE
                / Baseline2Model.LABOR_PRODUCTIVITY;
        double m = (Baseline2Model.HISTORICAL_LABOR_PRICE - minPrice)
                / quantity;
        double step = Math.max(Laborer.EAT_AMOUNT, quantity / 20.0);
        for (double x = 0.0; x < quantity; x += step) {
            double price = minPrice + m * x;
            // take avg of this price and last price
            price = (price + this.foodMarket.getLastMarketPrice()) / 2.0;
            // if (this.economy.getRunningTimeSteps() <= 10)
            // System.out.println("Step = "
            // + this.economy.getRunningTimeSteps()
            // + ", Farm offering: Price " + price);
            if (!this.foodMarket.addSellOffer(this, step, price))
                break;
        }

        // failed seeking algorithm
        // // sell food using a seek method
        // // nothing to sell
        // if (this.food.getQuantity() <= 0.0)
        // return;
        //
        // // initially sell at mp
        // if (this.foodMarket.getLastMarketPrice() <= 0.0) {
        // this.foodPrice = Baseline2Model.HISTORICAL_FOOD_PRICE;
        // if (this.food.getQuantity() > 0.0)
        // this.foodMarket.addSellOffer(this, this.food.getQuantity(),
        // this.foodPrice);
        // return;
        // }
        //
        // // basically sold all food last round, so try a higher price
        // if (this.food.getQuantity() - this.getLastAmountProduced() < 0.1 *
        // Laborer.EAT_AMOUNT) {
        // this.foodPrice += 0.01 * Baseline2Model.HISTORICAL_FOOD_PRICE;
        // System.out
        // .println("Farm Upping price to "
        // + this.foodPrice
        // + "(lastReal = "
        // + this.foodMarket.getLastMarketPrice()
        // + ") (myLast="
        // + (this.foodPrice + 0.01 * Baseline2Model.HISTORICAL_FOOD_PRICE)
        // + ")");
        // } else {
        // this.foodPrice -= 0.01 * Baseline2Model.HISTORICAL_FOOD_PRICE;
        // System.out
        // .println("Farm Lowering price to "
        // + this.foodPrice
        // + "(last = "
        // + this.foodMarket.getLastMarketPrice()
        // + ") (myLast="
        // + (this.foodPrice + 0.01 * Baseline2Model.HISTORICAL_FOOD_PRICE)
        // + ")");
        // }
        // this.foodPrice = Math.max(this.foodPrice,
        // Baseline2Model.HISTORICAL_LABOR_PRICE
        // / Baseline2Model.LABOR_PRODUCTIVITY);
        //
        // if (this.foodPrice > 0.0)
        // this.foodMarket.addSellOffer(this, this.food.getQuantity(),
        // this.foodPrice);
    }
}
