/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.*;
import eos.models.Laborer;
import static eos.models.Baseline3Model.*;

/**
 * A laborer implementation for the baseline 3 economy.
 * 
 * @author Michael Adelson
 * 
 */
public class BL3Laborer extends Baseline3Laborer {

    private static final double TARGET_FOOD_STOCK = 30.0,
            TARGET_MONEY_STOCK = 60.0;

    /**
     * @see Baseline3Laborer#Baseline3Laborer(Economy, Map)
     */
    public BL3Laborer(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * eos.models.Baseline3Model.Baseline3Laborer#baseLine3LaborerSubclassAct()
     */
    protected void baseLine3LaborerSubclassAct() {
        // buy food
        if (this.foodMarket.getLastMarketPrice() <= 0.0
                || this.laborMarket.getLastMarketPrice() <= 0.0)
            this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                    HISTORICAL_FOOD_PRICE);
        else {
            final double quantum = Laborer.EAT_AMOUNT * 2.0; // Laborer.EAT_AMOUNT
            // / 2.0;
            double m = (this.foodMarket.getLastMarketPrice() - this.laborMarket
                    .getLastMarketPrice())
                    / TARGET_FOOD_STOCK;
            double b = this.laborMarket.getLastMarketPrice();

            if (m > 0.0)
                this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                        this.foodMarket.getLastMarketPrice());

            for (double x = this.food.getQuantity(); x <= 1.5 * TARGET_FOOD_STOCK; x += quantum) {
                double price = m * x + b;
                if (price <= 0.0)
                    break;
                this.foodMarket.addBuyOffer(this, quantum, price);
            }
        }
        
        // sell labor
        double price;
        if (this.laborMarket.getLastMarketPrice() <= 0.0)
            price = getHistoricalPrice(this.labor);
        else {
            final double frac = 0.95, range = 1.0 / frac - frac;
            price = this.laborMarket.getLastMarketPrice()
                    * (this.economy.random() * range + frac);
        }

        if (this.money.getQuantity() > TARGET_MONEY_STOCK
                && this.food.getQuantity() > TARGET_FOOD_STOCK) {
            // return;
            double quantity = (this.money.getQuantity() - TARGET_MONEY_STOCK)
                    / getHistoricalPrice(this.utility);
            this.utilityMarket.addBuyOffer(this, quantity,
                    getHistoricalPrice(this.utility));
            price *= 2.0; // raise price since the laborer doesn't NEED to work
        }

        this.laborMarket.addSellOffer(this, Laborer.LABOR_ALLOWANCE, price);
    }
}
