/**
 * 
 */
package eos.agents;

import eos.models.BaselineModel.*;
import eos.framework.*;
import eos.utilities.*;
import java.util.*;

/**
 * A laborer who asks and sells for a fixed price.
 * 
 * @author Madelson
 */
public class FixedPriceLaborer extends BaselineLaborer {

    public static final double PRICE_RATIO = 2; // 2.5
    public static final double FOOD_PRICE = 0.2;
    public static final double LABOR_PRICE = PRICE_RATIO * FOOD_PRICE
            * EAT_AMOUNT / LABOR_ALLOWANCE;
    public static final double FOOD_THRESHOLD = 30;

    /**
     * The laborer's markets.
     */
    private Market foodMarket, laborMarket;

    /**
     * Construct a FixedPriceLaborer. Must have first added Markets to economy.
     */
    public FixedPriceLaborer(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        foodMarket = this.economy.getMarketFor(this.money.getName(), this.food
                .getName());
        laborMarket = this.economy.getMarketFor(this.money.getName(),
                this.labor.getName());

        Errors.Assert(foodMarket != null && laborMarket != null,
                "Markets must be added to economy BEFORE FixedPriceLaborers");
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.models.Laborer#bid()
     */
    protected void bid() {
        if (this.food.getQuantity() < FOOD_THRESHOLD) {
            // buy food
            foodMarket.addBuyOffer(this, FOOD_THRESHOLD - food.getQuantity(),
                    FOOD_PRICE);

            // sell labor
            double laborQuantity = (FOOD_THRESHOLD - food.getQuantity())
                    * FOOD_PRICE / LABOR_PRICE;
            laborQuantity = Math.min(LABOR_ALLOWANCE, laborQuantity);

            // if (laborQuantity >= 0.5 * LABOR_ALLOWANCE)
            laborMarket.addSellOffer(this, laborQuantity, LABOR_PRICE);
        }
//        if (this.economy.getRandom().nextInt(10) == 0
//                && this.labor.getQuantity() > 0.0) { // a one in ten chance
//            laborMarket
//                    .addSellOffer(this, this.labor.getQuantity(), LABOR_PRICE);
//        }
    }

}
