/**
 * 
 */
package eos.goods;

import java.util.HashMap;

import eos.framework.Good;
import eos.utilities.Errors;

/**
 * Represents an empty (Quantity = 0) Good.
 * 
 * @author Michael Adelson
 */
public class EmptyGood implements Good {

    /**
     * Holds all instantiations of EmptyGood.
     */
    private static final HashMap<String, EmptyGood> emptyGoods = new HashMap<String, EmptyGood>();

    /**
     * @return the EmptyGood associated with name.
     */
    public static final EmptyGood get(String name) {
        EmptyGood eg = emptyGoods.get(name);
        if (eg != null)
            return eg;
        eg = new EmptyGood(name);
        emptyGoods.put(name, eg);
        
        return eg;
    }
    
    /**
     * The name of this Good.
     */
    private final String name;

    /**
     * Use EmptyGood.get();
     */
    private EmptyGood(String name) {
        this.name = name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.framework.Good#deposit(double)
     */
    public final void deposit(double amount) {
        Errors.err("Can't deposit into an empty Good");
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.framework.Good#getName()
     */
    public final String getName() {
        return this.name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.framework.Good#getQuantity()
     */
    public final double getQuantity() {
        return 0;
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.framework.Good#isDiscrete()
     */
    public boolean isDiscrete() {
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.framework.Good#withdraw(double)
     */
    public final double withdraw(double amount) {
        return 0;
    }

}
