package eos.io;

import eos.framework.*;

import java.util.*;

/**
 * This interface defines a set of DataOutput methods for the Economy class.
 * This allows Simulations and Printers to have access to various components of
 * the economy without giving that same access to Agents.
 * 
 * @author Michael Adelson
 */
public interface DataOutputEconomy {

    /**
     * @return A Collection view of all the Markets in the IOEconomy.
     */
    public Collection<Agent> getAgents();

    /**
     * @returns An Collection view of all the Markets in the IOEconomy.
     */
    public Collection<? extends Market> getMarkets();

    /**
     * @returns a reference to the Market in this Economy that uses currency and
     *          product. This method should return NULL if no such Market
     *          exists.
     */
    public Market getMarketFor(String currency, String product);

    /**
     * @returns The number of time steps that the simulation has run (starts at
     *          0.
     */
    public int getRunningTimeSteps();

    /**
     * Add printer to this DataOutPutEconomy so that its print() and title()
     * methods are called every timestep. Note that this is only one way of
     * printing. Simulation classes may instead prefer to call print themselves.
     * Also note that it is up to the calling class to eventually call printer's
     * cleanup() method.
     */
    public void addPrinter(Printer printer);
    
    /**
     * Calls Printer.cleanup(this) for each Printer in this DataOutputEconomy.
     */
    public void cleanupPrinters();
}
