package eos.io;

/**
 * A class which retrieves data from an Economy. This data is returned to and
 * written to a file by a Printer object.
 * 
 * @author madelson
 */
public interface DataPrinter {
    
    /**
     * Return a title for this DataPrinter.
     */
    String getTitle(DataOutputEconomy economy);
    
    /**
     * Return the next piece of data for this DataPrinter.
     */
    String getData(DataOutputEconomy economy);
    
    /**
     * Return any final data (available on cleanup) for this DataPrinter.
     * This method should also remove any references to economy for the purposes
     * of garbage collection.
     */
    String getCleanupData(DataOutputEconomy economy);  
    
    /**
     * Can this printer's output legally be put in a CSV cell?
     */
    boolean isCSVCompatible();
    
    /**
     * Can this printer be correctly called if the economy supplied to getData()
     * is not always the same?
     */
    boolean isMultiRunCompatible();
}
