/**
 * 
 */
package eos.simulations;

import eos.agents.*;
import eos.economies.BL321Economy;
import eos.framework.Economy;
import eos.io.GoodTrackerDataPrinter;
import eos.io.Simulation;
import eos.io.Simulation.InitializationFunction;
import eos.markets.*;
import eos.models.Baseline2Model;
import eos.models.Baseline2Model.Baseline2Good;

/**
 * @author madelson
 *
 */
public class DynamicFoodSim {
    
    // parameters
    static int numFarms = 2;
    static int numLaborers = 58 * numFarms;
    
    static Simulation sim;
    
    static {
        sim = new Simulation("DynamicFood", BL321Economy.class, 578294307504387L/*new Random().nextLong()*/);
        sim.addGoods(Baseline2Good.class, "Food", "Labor", "Money", "Leisure");
        
        sim.addMarkets(CallAuctionMarket.class, "Money", "Food", "Labor");
        sim.addAgents(BL2FoodDemandCurveOL.class, numLaborers);
        sim.addOwnersAndFirms(BL2FoodDemandCurveOL.class, numFarms, BL2FoodSupplyCurveFarm.class, 1);
        
        sim.setQuantities(BL2FoodDemandCurveOL.class, 40, 0, 60, 0);
        sim.setQuantities(BL2FoodSupplyCurveFarm.class, 20, 0, 100, 0);
        
        // set the prices before running
        sim.addInitializationFunction(new InitializationFunction() {
            public void initialize(Economy economy) {
                Baseline2Model.HISTORICAL_LABOR_PRICE = (59.0 / 24.0)
                * Baseline2Model.HISTORICAL_FOOD_PRICE;
            }
        });
    }
    
    /**
     * @param args
     */
    public static void main(String[] args) {
        //sim.setDefaultPrintPeriod(1);
        sim.setPerturbation(10);
        sim.addDataPrinter(new GoodTrackerDataPrinter("Money", false));
        long clock = System.currentTimeMillis();
        sim.run(20000);
        System.out.println(System.currentTimeMillis() - clock);
    }

}
